@@ -1,8 +1,9 @@
-import copy
 import json
 import pathlib
 from functools import reduce
+from glob import has_magic
 from numbers import Number
+from packaging.version import Version
 from typing import Any, Dict, Iterable, List, Optional, Tuple, Union
 
 import fsspec
@@ -22,7 +23,14 @@ from ..geometry import (GeometryDtype, LineDtype, MultiLineDtype,
                                     MultiPointDtype, MultiPolygonDtype,
                                     PointDtype, PolygonDtype, RingDtype)
 from ..geometry.base import to_geometry_array
-from ..io.utils import validate_coerce_filesystem
+from ..io.utils import (
+    PathType,
+    _maybe_prepend_protocol,
+    validate_coerce_filesystem,
+)
+
+# improve pandas compatibility, based on geopandas _compat.py
+PANDAS_GE_12 = Version(pd.__version__) >= Version("1.2.0")
 
 _geometry_dtypes = [
     PointDtype, MultiPointDtype, RingDtype, LineDtype,
@@ -39,22 +47,33 @@ def _import_geometry_columns(df, geom_cols):
     return df.assign(**new_cols)
 
 
-def _load_parquet_pandas_metadata(path, filesystem=None):
-    filesystem = validate_coerce_filesystem(path, filesystem)
+def _load_parquet_pandas_metadata(
+    path,
+    filesystem=None,
+    storage_options=None,
+    engine_kwargs=None,
+):
+    engine_kwargs = engine_kwargs or {}
+    filesystem = validate_coerce_filesystem(path, filesystem, storage_options)
     if not filesystem.exists(path):
         raise ValueError("Path not found: " + path)
 
     if filesystem.isdir(path):
         pqds = pq.ParquetDataset(
-            path, filesystem=filesystem, validate_schema=False
+            path,
+            filesystem=filesystem,
+            #validate_schema=False,
+            use_legacy_dataset=False,
+            **engine_kwargs,
         )
-        common_metadata = pqds.common_metadata
-        if common_metadata is None:
-            # Get metadata for first piece
-            piece = pqds.pieces[0]
-            metadata = piece.get_metadata().metadata
-        else:
-            metadata = pqds.common_metadata.metadata
+        filename = pathlib.Path(pqds.files[0]).parent.joinpath("_common_metadata")
+        try:
+            common_metadata = pq.read_metadata(filename)
+        except FileNotFoundError:
+            # Common metadata doesn't exist, so get metadata for first piece instead
+            filename = pathlib.Path(pqds.files[0])
+            common_metadata = pq.read_metadata(filename)
+        metadata = common_metadata.metadata
     else:
         with filesystem.open(path) as f:
             pf = pq.ParquetFile(f)
@@ -85,31 +104,55 @@ def _get_geometry_columns(pandas_metadata):
 
 def to_parquet(
     df: GeoDataFrame,
-    fname,
+    path: PathType,
     compression: Optional[str] = "snappy",
+    filesystem: Optional[fsspec.AbstractFileSystem] = None,
     index: Optional[bool] = None,
-    **kwargs,
+    storage_options: Optional[Dict[str, Any]] = None,
+    **kwargs: Any,
 ) -> None:
+    if filesystem is not None:
+        filesystem = validate_coerce_filesystem(path, filesystem, storage_options)
+
     # Standard pandas to_parquet with pyarrow engine
-    pd_to_parquet(
-        df,
-        fname,
-        engine="pyarrow",
-        compression=compression,
-        index=index,
+    to_parquet_args = {
+        "df": df,
+        "path": path,
+        "engine": "pyarrow",
+        "compression": compression,
+        "filesystem": filesystem,
+        "index": index,
         **kwargs,
-    )
+    }
+
+    if PANDAS_GE_12:
+        to_parquet_args.update({"storage_options": storage_options})
+    else:
+        if filesystem is None:
+            filesystem = validate_coerce_filesystem(path, filesystem, storage_options)
+            to_parquet_args.update({"filesystem": filesystem})
+
+    pd_to_parquet(**to_parquet_args)
 
 
 def read_parquet(
-    path: str,
+    path: PathType,
     columns: Optional[Iterable[str]] = None,
-    filesystem: Optional[fsspec.spec.AbstractFileSystem] = None,
+    filesystem: Optional[fsspec.AbstractFileSystem] = None,
+    storage_options: Optional[Dict[str, Any]] = None,
+    engine_kwargs: Optional[Dict[str, Any]] = None,
+    **kwargs: Any,
 ) -> GeoDataFrame:
-    filesystem = validate_coerce_filesystem(path, filesystem)
+    engine_kwargs = engine_kwargs or {}
+    filesystem = validate_coerce_filesystem(path, filesystem, storage_options)
 
     # Load pandas parquet metadata
-    metadata = _load_parquet_pandas_metadata(path, filesystem=filesystem)
+    metadata = _load_parquet_pandas_metadata(
+        path,
+        filesystem=filesystem,
+        storage_options=storage_options,
+        engine_kwargs=engine_kwargs,
+    )
 
     # If columns specified, prepend index columns to it
     if columns is not None:
@@ -130,7 +173,11 @@ def read_parquet(
 
     # Load using pyarrow to handle parquet files and directories across filesystems
     df = pq.ParquetDataset(
-        path, filesystem=filesystem, validate_schema=False
+        path,
+        filesystem=filesystem,
+        validate_schema=False,
+        **engine_kwargs,
+        **kwargs,
     ).read(columns=columns).to_pandas()
 
     # Import geometry columns, not needed for pyarrow >= 0.16
@@ -144,64 +191,57 @@ def read_parquet(
 
 def to_parquet_dask(
     ddf: DaskGeoDataFrame,
-    path,
+    path: PathType,
     compression: Optional[str] = "snappy",
-    filesystem: Optional[fsspec.spec.AbstractFileSystem] = None,
+    filesystem: Optional[fsspec.AbstractFileSystem] = None,
     storage_options: Optional[Dict[str, Any]] = None,
-    **kwargs,
+    engine_kwargs: Optional[Dict[str, Any]] = None,
+    **kwargs: Any,
 ) -> None:
-    assert isinstance(ddf, DaskGeoDataFrame)
-    filesystem = validate_coerce_filesystem(path, filesystem)
-    if path and filesystem.isdir(path):
+    engine_kwargs = engine_kwargs or {}
+
+    if not isinstance(ddf, DaskGeoDataFrame):
+        raise TypeError(f"Expected DaskGeoDataFrame not {type(ddf)}")
+    filesystem = validate_coerce_filesystem(path, filesystem, storage_options)
+    if kwargs.get("overwrite", False) and \
+        not kwargs.get("append", False) and \
+            path and filesystem.isdir(path):
         filesystem.rm(path, recursive=True)
 
+    # Determine partition bounding boxes to save to _metadata file
+    partition_bounds = {}
+    for series_name in ddf.columns:
+        series = ddf[series_name]
+        if isinstance(series.dtype, GeometryDtype):
+            partition_bounds[series_name] = series.partition_bounds.to_dict()
+
+    spatial_metadata = {'partition_bounds': partition_bounds}
+    b_spatial_metadata = json.dumps(spatial_metadata).encode('utf')
+
     dd_to_parquet(
         ddf,
         path,
         engine="pyarrow",
         compression=compression,
         storage_options=storage_options,
+        custom_metadata={b'spatialpandas': b_spatial_metadata},
+        write_metadata_file=True,
+        **engine_kwargs,
         **kwargs,
     )
 
-    # Write partition bounding boxes to the _metadata file
-    partition_bounds = {}
-    for series_name in ddf.columns:
-        series = ddf[series_name]
-        if isinstance(series.dtype, GeometryDtype):
-            if series._partition_bounds is None:
-                # Bounds are not already computed. Compute bounds from the parquet file
-                # that was just written.
-                filesystem.invalidate_cache(path)
-                series = read_parquet_dask(
-                    path,
-                    columns=[series_name],
-                    filesystem=filesystem,
-                    load_divisions=False
-                )[series_name]
-            partition_bounds[series_name] = series.partition_bounds.to_dict()
-
-    spatial_metadata = {'partition_bounds': partition_bounds}
-    b_spatial_metadata = json.dumps(spatial_metadata).encode('utf')
-
-    pqds = pq.ParquetDataset(path, filesystem=filesystem, validate_schema=False)
-    all_metadata = copy.copy(pqds.common_metadata.metadata)
-    all_metadata[b'spatialpandas'] = b_spatial_metadata
-    schema = pqds.common_metadata.schema.to_arrow_schema()
-    new_schema = schema.with_metadata(all_metadata)
-    with filesystem.open(pqds.common_metadata_path, 'wb') as f:
-        pq.write_metadata(new_schema, f)
-
 
 def read_parquet_dask(
-    path: str,
+    path: PathType,
     columns: Optional[Iterable[str]] = None,
-    filesystem: Optional[fsspec.spec.AbstractFileSystem] = None,
+    filesystem: Optional[fsspec.AbstractFileSystem] = None,
     load_divisions: Optional[bool] = False,
     geometry: Optional[str] = None,
     bounds: Optional[Tuple[Number, Number, Number, Number]] = None,
     categories: Optional[Union[List[str], Dict[str, str]]] = None,
     build_sindex: Optional[bool] = False,
+    storage_options: Optional[Dict[str, Any]] = None,
+    engine_kwargs: Optional[Dict[str, Any]] = None,
 ) -> DaskGeoDataFrame:
     """Read spatialpandas parquet dataset(s) as DaskGeoDataFrame.
 
@@ -232,30 +272,42 @@ def read_parquet_dask(
             data written by dask/fastparquet, not otherwise.
         build_sindex : boolean
             Whether to build partition level spatial indexes to speed up indexing.
+        storage_options: Key/value pairs to be passed on to the file-system backend, if any.
+        engine_kwargs: pyarrow.parquet engine-related keyword arguments.
     Returns:
     DaskGeoDataFrame
     """
     # Normalize path to a list
     if isinstance(path, (str, pathlib.Path)):
-        path = [str(path)]
+        paths = [str(path)]
     else:
-        path = list(path)
-        if not path:
+        paths = list(path)
+        if not paths:
             raise ValueError('Empty path specification')
 
     # Infer filesystem
-    filesystem = validate_coerce_filesystem(path[0], filesystem)
+    filesystem = validate_coerce_filesystem(
+        paths[0],
+        filesystem,
+        storage_options,
+    )
 
     # Expand glob
-    if len(path) == 1 and ('*' in path[0] or '?' in path[0] or '[' in path[0]):
-        path = filesystem.glob(path[0])
-        path = _maybe_prepend_protocol(path, filesystem)
+    if len(paths) == 1 and has_magic(paths[0]):
+        paths = filesystem.glob(paths[0])
+        paths = _maybe_prepend_protocol(paths, filesystem)
 
     # Perform read parquet
     result = _perform_read_parquet_dask(
-        path, columns, filesystem,
-        load_divisions=load_divisions, geometry=geometry, bounds=bounds,
-        categories=categories
+        paths,
+        columns,
+        filesystem,
+        load_divisions=load_divisions,
+        geometry=geometry,
+        bounds=bounds,
+        categories=categories,
+        storage_options=storage_options,
+        engine_kwargs=engine_kwargs,
     )
 
     if build_sindex:
@@ -264,36 +316,47 @@ def read_parquet_dask(
     return result
 
 
-def _maybe_prepend_protocol(paths, filesystem):
-    protocol = filesystem.protocol if isinstance(
-        filesystem.protocol, str) else filesystem.protocol[0]
-    if protocol not in ("file", "abstract"):
-        # Add back prefix (e.g. s3://)
-        paths = [
-            "{proto}://{p}".format(proto=protocol, p=p) for p in paths
-        ]
-    return paths
-
-
 def _perform_read_parquet_dask(
-        paths, columns, filesystem, load_divisions, geometry=None, bounds=None,
-        categories=None,
+    paths,
+    columns,
+    filesystem,
+    load_divisions,
+    geometry=None,
+    bounds=None,
+    categories=None,
+    storage_options=None,
+    engine_kwargs=None,
 ):
-    filesystem = validate_coerce_filesystem(paths[0], filesystem)
-    datasets = [pa.parquet.ParquetDataset(
-        path, filesystem=filesystem, validate_schema=False
-    ) for path in paths]
+    engine_kwargs = engine_kwargs or {}
+    filesystem = validate_coerce_filesystem(
+        paths[0],
+        filesystem,
+        storage_options,
+    )
+    datasets = [
+        pa.parquet.ParquetDataset(
+            path,
+            filesystem=filesystem,
+            #validate_schema=False,
+            use_legacy_dataset=False,
+            **engine_kwargs,
+        ) for path in paths
+    ]
 
     # Create delayed partition for each piece
     pieces = []
     for dataset in datasets:
         # Perform natural sort on pieces so that "part.10" comes after "part.2"
-        dataset_pieces = sorted(dataset.pieces, key=lambda piece: natural_sort_key(piece.path))
+        dataset_pieces = sorted(dataset.fragments, key=lambda piece: natural_sort_key(piece.path))
         pieces.extend(dataset_pieces)
 
     delayed_partitions = [
         delayed(read_parquet)(
-            piece.path, columns=columns, filesystem=filesystem
+            piece.path,
+            columns=columns,
+            filesystem=filesystem,
+            storage_options=storage_options,
+            engine_kwargs=engine_kwargs,
         )
         for piece in pieces
     ]
@@ -337,16 +400,23 @@ def _perform_read_parquet_dask(
         cols_no_index = None
 
     meta = dd_read_parquet(
-        paths[0],
+        datasets[0].files[0],
         columns=cols_no_index,
         filesystem=filesystem,
         engine='pyarrow',
         categories=categories,
         gather_statistics=False,
+        storage_options=storage_options,
+        **engine_kwargs,
     )._meta
 
     # Import geometry columns in meta, not needed for pyarrow >= 0.16
-    metadata = _load_parquet_pandas_metadata(paths[0], filesystem=filesystem)
+    metadata = _load_parquet_pandas_metadata(
+        paths[0],
+        filesystem=filesystem,
+        storage_options=storage_options,
+        engine_kwargs=engine_kwargs,
+    )
     geom_cols = _get_geometry_columns(metadata)
     if geom_cols:
         meta = _import_geometry_columns(meta, geom_cols)
@@ -425,10 +495,15 @@ def _perform_read_parquet_dask(
 
 def _load_partition_bounds(pqds):
     partition_bounds = None
-    if (pqds.common_metadata is not None and
-            b'spatialpandas' in pqds.common_metadata.metadata):
+    filename = pathlib.Path(pqds.files[0]).parent.joinpath("_common_metadata")
+    try:
+        common_metadata = pq.read_metadata(filename)
+    except FileNotFoundError:
+        common_metadata = None
+
+    if common_metadata is not None and b'spatialpandas' in common_metadata.metadata:
         spatial_metadata = json.loads(
-            pqds.common_metadata.metadata[b'spatialpandas'].decode('utf')
+            common_metadata.metadata[b'spatialpandas'].decode('utf')
         )
         if "partition_bounds" in spatial_metadata:
             partition_bounds = {}
